from im2dhisteq import im2dhisteq
from im2dhist import im2dhist
import cv2
import numpy as np

def test_im2dhisteq_with_param():
    image_name = '../assets/Plane.jpg'
    image = cv2.imread(image_name)
    # convert rgb image to gray
    gray_image = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
    
    gray_image_2DHisteq = im2dhisteq(gray_image, w_neighboring=6)
    

    #np.save(f'{image_name}-2D-Histogram-Equalized', gray_image_2DHisteq)
    gray_image_2DHisteq_cmpr = np.load(f'{image_name}-2D-Histogram-Equalized.npy')
    assert np.all(gray_image_2DHisteq == gray_image_2DHisteq_cmpr)
#test_im2dhisteq_with_param()



from im2dhist import im2dhist, imhist
import numpy as np
import numba 

@numba.njit()
def im2dhisteq(image, w_neighboring=6):
    V = image.copy()
    [h, w] = V.shape
    V_hist = imhist(V)
    H_in = im2dhist(V, w_neighboring=w_neighboring)
    CDFx = np.cumsum(np.sum(H_in, axis=0)) # Kx1

    # normalizes CDFx
    CDFxn = (255*CDFx/CDFx[-1])

    PDFxn = np.zeros_like(CDFxn)
    PDFxn[0] = CDFxn[0]
    PDFxn[1:] = np.diff(CDFxn)

    X_transform = np.zeros((256))
    X_transform[np.where(V_hist > 0)] = PDFxn.copy()
    CDFxn_transform = np.cumsum(X_transform)


    # bins = np.array([i for i in range(0, 256)])
    bins = np.arange(256)
    # uses linear interpolation of cdf to find new pixel values
    image_equalized = np.floor(np.interp(V.flatten(), bins, CDFxn_transform).reshape(h, w)).astype(np.uint8)

    return image_equalized

@numba.njit()
def vid2dhisteq(image, w_neighboring=6, Wout_list=np.zeros((10))):
    [h, w] = image.shape
    V = image.copy()
    V_hist = imhist(V)

    H_in = im2dhist(V, w_neighboring=6)

    CDFx = np.cumsum(np.sum(H_in, axis=0)) # Kx1

    # normalizes CDFx
    CDFxn = (CDFx/CDFx[-1])

    PDFxn = np.zeros_like(CDFxn)
    PDFxn[0] = CDFxn[0]
    PDFxn[1:] = np.diff(CDFxn)
    
    X_transform = np.zeros((256))
    X_transform[np.where(V_hist > 0)] = PDFxn.copy()
    CDFxn_transform = np.cumsum(X_transform).copy()

    Win = H_in.shape[0]
    Gmax = 1.5 # 1.5 .. 2
    Wout = min(255, Gmax*Win)
    if np.where(Wout_list>0)[0].size==Wout_list.size:
        Wout = (np.sum(Wout_list)+Wout) / (1+Wout_list.size)

    F = V.copy().reshape(-1)
    Ftilde = Wout * CDFxn_transform[F]
    
    Madj = np.mean(V) - np.mean(Ftilde)
    Ftilde = Ftilde + Madj

    Ftilde = np.where(Ftilde>=0, Ftilde, np.zeros_like(Ftilde))
    Ftilde = np.where(Ftilde<=255, Ftilde, 255*np.ones_like(Ftilde))
    Ftilde = Ftilde.astype(np.uint8)

    image_equalized = Ftilde.reshape(h, w)

    return image_equalized, Wout



from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()


setup(
    name='im2dhisteq',
    version='0.0.11',
    author="mamdasn s",
    author_email="<mamdassn@gmail.com>",
    url="https://github.com/Mamdasn/im2dhisteq",
    description='This module attempts to enhance contrast of a given image by equalizing its two dimensional histogram.',
    long_description=long_description,
    long_description_content_type = "text/markdown",
    include_package_data=True,
    package_dir={'': 'src'},
    py_modules=["im2dhisteq"],
    install_requires=[
        "numpy", 
        "numba",
        "im2dhist",
        ],
    keywords=['python', 'histogram', 'imhist', '2dhist', 'hist2d', 'im2dhisteq', 'histogram equalization', 'two dimensional histogram'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
    ],
    scripts=['bin/im2dhisteq'],
)
